@@ -21,30 +21,35 @@
  * 02111-1307, USA.
  */
 
+#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
+
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include "showdesktop.h"
+#endif
+
+#include <gtk/gtkicontheme.h>
 #include <gtk/gtktogglebutton.h>
-#include <gtk/gtktooltips.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkmessagedialog.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-about.h>
-#include <libgnomeui/gnome-icon-theme.h>
-#define WNCK_I_KNOW_THIS_IS_UNSTABLE
-#include <libwnck/screen.h>
-#include "egg-screen-help.h"
-#include "wncklet.h"
+#include <gtk/gtkdnd.h>
 #include <gdk/gdkx.h>
 
+#include <libwnck/screen.h>
+
+#include "wncklet.h"
+#include "showdesktop.h"
+
 #include <string.h>
 
+#define TIMEOUT_ACTIVATE 1000
+
+
 typedef struct {
         /* widgets */
         GtkWidget *applet;
         GtkWidget *button;
         GtkWidget *image;
-        GdkPixbuf *icon;
         GtkWidget *about_dialog;
 
         PanelAppletOrient orient;
@@ -53,8 +58,9 @@ typedef struct {
         WnckScreen *wnck_screen;
 
         guint showing_desktop : 1;
+        guint button_activate;
 
-	GnomeIconTheme *icon_theme;
+	GtkIconTheme *icon_theme;
 } ShowDesktopData;
 
 static void display_help_dialog  (BonoboUIComponent *uic,
@@ -68,35 +74,20 @@ static void update_icon           (ShowDesktopData *sdd);
 static void update_button_state   (ShowDesktopData *sdd);
 static void update_button_display (ShowDesktopData *sdd);
 
+static void theme_changed_callback        (GtkIconTheme    *icon_theme,
+					   ShowDesktopData *sdd);
+
 static void button_toggled_callback       (GtkWidget       *button,
                                            ShowDesktopData *sdd);
 static void show_desktop_changed_callback (WnckScreen      *screen,
                                            ShowDesktopData *sdd);
 
-static void
-set_tooltip (GtkWidget  *widget,
-             const char *tip)
-{
-        GtkTooltips *tooltips;
-
-        tooltips = g_object_get_data (G_OBJECT (widget), "tooltips");
-        if (!tooltips) {
-                tooltips = gtk_tooltips_new ();
-                g_object_ref (tooltips);
-                gtk_object_sink (GTK_OBJECT (tooltips));
-                g_object_set_data_full (G_OBJECT (widget), "tooltips", tooltips,
-                                        (GDestroyNotify) g_object_unref);
-        }
-
-        gtk_tooltips_set_tip (tooltips, widget, tip, NULL);
-}
-
 /* this is when the panel orientation changes */
 
 static void
 applet_change_orient (PanelApplet       *applet,
                       PanelAppletOrient  orient,
-                      ShowDesktopData         *sdd)
+                      ShowDesktopData   *sdd)
 {
         GtkOrientation new_orient;
 
@@ -121,41 +112,26 @@ applet_change_orient (PanelApplet       *applet,
         update_icon (sdd);
 }
 
-static void
-applet_change_background (PanelApplet               *applet,
-                          PanelAppletBackgroundType  type,
-                          GdkColor                  *color,
-                          const gchar               *pixmap,
-                          ShowDesktopData                 *sdd)
-{
-        if (type == PANEL_NO_BACKGROUND) {
-                GtkRcStyle *rc_style;
-
-                rc_style = gtk_rc_style_new ();
-
-                gtk_widget_modify_style (sdd->applet, rc_style);
-
-                g_object_unref (rc_style);
-
-        } else if (type == PANEL_COLOR_BACKGROUND)
-                gtk_widget_modify_bg (sdd->applet, GTK_STATE_NORMAL, color);
-
-        /* else if (type == PANEL_PIXMAP_BACKGROUND)
-         * FIXME: Handle this when the panel support works again
-         */
-}
-
-
 /* this is when the panel size changes */
 static void
-applet_change_pixel_size (PanelApplet *applet,
-                          gint         size,
-                          ShowDesktopData   *sdd)
+applet_size_allocated (PanelApplet       *applet,
+		       GtkAllocation     *allocation,
+                       ShowDesktopData   *sdd)
 {
-        if (sdd->size == size)
-                return;
+	if (((sdd->orient == GTK_ORIENTATION_HORIZONTAL)
+		&& (sdd->size == allocation->height))
+	    || ((sdd->orient == GTK_ORIENTATION_VERTICAL)
+	    	&& (sdd->size == allocation->width)))
+	     return;
 
-        sdd->size = size;
+	switch (sdd->orient) {
+	case GTK_ORIENTATION_HORIZONTAL:
+		sdd->size = allocation->height;
+		break;
+	case GTK_ORIENTATION_VERTICAL:
+		sdd->size = allocation->width;
+		break;
+	}
 
         update_icon (sdd);
 }
@@ -164,46 +140,66 @@ static void
 update_icon (ShowDesktopData *sdd)
 {
         int width, height;
+        GdkPixbuf *icon;
         GdkPixbuf *scaled;
-        double aspect;
         int icon_size;
+	GError *error;
 
-        if (sdd->icon == NULL)
-                return;
+	if (!sdd->icon_theme)
+		return;
 
-        width = gdk_pixbuf_get_width (sdd->icon);
-        height = gdk_pixbuf_get_height (sdd->icon);
+#define SPACE_FOR_BUTTON_BORDER 4
+	icon_size = sdd->size - SPACE_FOR_BUTTON_BORDER;
 
-        aspect = (double) width / (double) height;
+	error = NULL;
+	icon = gtk_icon_theme_load_icon (sdd->icon_theme,
+					 "gnome-fs-desktop",
+					 icon_size, 0, &error);
 
-#define SPACE_FOR_BUTTON_BORDER 3
-        icon_size = sdd->size - SPACE_FOR_BUTTON_BORDER;
+	if (icon == NULL) {
+		g_printerr (_("Failed to load %s: %s\n"), "gnome-fs-desktop",
+			    error ? error->message : _("Icon not found"));
+		if (error) {
+			g_error_free (error);
+			error = NULL;
+		}
+
+		gtk_image_set_from_stock (GTK_IMAGE (sdd->image),
+					  GTK_STOCK_MISSING_IMAGE,
+					  GTK_ICON_SIZE_SMALL_TOOLBAR);
+		return;
+	}
+
+        width = gdk_pixbuf_get_width (icon);
+        height = gdk_pixbuf_get_height (icon);
 
         scaled = NULL;
 
         /* Make it fit on the given panel */
         switch (sdd->orient) {
         case GTK_ORIENTATION_HORIZONTAL:
-                width = icon_size * aspect;
+                width = (icon_size * width) / height;
                 height = icon_size;
                 break;
         case GTK_ORIENTATION_VERTICAL:
-                height = icon_size / aspect;
+                height = (icon_size * height) / width;
                 width = icon_size;
                 break;
         }
 
-        scaled = gdk_pixbuf_scale_simple (sdd->icon,
+        scaled = gdk_pixbuf_scale_simple (icon,
                                           width, height,
                                           GDK_INTERP_BILINEAR);
 
-        if (scaled == NULL)
-                return;
+        if (scaled != NULL) {
+		gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
+					   scaled);
+		g_object_unref (scaled);
+	} else
+		gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
+					   icon);
 
-        gtk_image_set_from_pixbuf (GTK_IMAGE (sdd->image),
-                                   scaled);
-
-        g_object_unref (G_OBJECT (scaled));
+        g_object_unref (icon);
 }
 
 static const BonoboUIVerb show_desktop_menu_verbs [] = {
@@ -219,11 +215,11 @@ static void
 update_button_display (ShowDesktopData *sdd)
 {
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sdd->button))) {
-                set_tooltip (sdd->button,
-                             _("Click here to restore hidden windows."));
+                wncklet_set_tooltip (sdd->button,
+				     _("Click here to restore hidden windows."));
         } else {
-                set_tooltip (sdd->button,
-                             _("Click here to hide all windows and show the desktop."));
+                wncklet_set_tooltip (sdd->button,
+				     _("Click here to hide all windows and show the desktop."));
         }
 }
 
@@ -257,14 +253,30 @@ static void
 applet_destroyed (GtkWidget       *applet,
                   ShowDesktopData *sdd)
 {
-        g_object_unref (G_OBJECT (sdd->icon));
-	g_object_unref (sdd->icon_theme);
-
 	if (sdd->about_dialog) {
 		gtk_widget_destroy (sdd->about_dialog);
 		sdd->about_dialog =  NULL;
 	}
 
+	if (sdd->button_activate != 0) {
+		g_source_remove (sdd->button_activate);
+		sdd->button_activate = 0;
+	}
+
+	if (sdd->wnck_screen != NULL) {
+		g_signal_handlers_disconnect_by_func (sdd->wnck_screen,
+						      show_desktop_changed_callback,
+						      sdd);
+		sdd->wnck_screen = NULL;
+	}
+
+	if (sdd->icon_theme != NULL) {
+		g_signal_handlers_disconnect_by_func (sdd->icon_theme,
+						      theme_changed_callback,
+						      sdd);
+		sdd->icon_theme = NULL;
+	}
+
         g_free (sdd);
 }
 
@@ -279,13 +291,102 @@ do_not_eat_button_press (GtkWidget      *widget,
         return FALSE;
 }
 
+static gboolean
+button_motion_timeout (gpointer data)
+{
+	ShowDesktopData *sdd = (ShowDesktopData*) data;
+
+	sdd->button_activate = 0;
+
+	g_signal_emit_by_name (G_OBJECT (sdd->button), "clicked", sdd);
+
+	return FALSE;
+}
+
+static void
+button_drag_leave (GtkWidget          *widget,
+		   GdkDragContext     *context,
+		   guint               time,
+		   ShowDesktopData    *sdd)
+{
+	if (sdd->button_activate != 0) {
+		g_source_remove (sdd->button_activate);
+		sdd->button_activate = 0;
+	}
+}
+
+static gboolean
+button_drag_motion (GtkWidget          *widget,
+		    GdkDragContext     *context,
+		    gint                x,
+		    gint                y,
+		    guint               time,
+		    ShowDesktopData    *sdd)
+{
+
+	if (sdd->button_activate == 0)
+		sdd->button_activate = g_timeout_add (TIMEOUT_ACTIVATE,
+						      button_motion_timeout,
+						      sdd);
+	gdk_drag_status (context, 0, time);
+	
+	return TRUE;
+}
+
+static void 
+show_desktop_applet_realized (PanelApplet *applet, 
+			      gpointer     data)
+{
+	ShowDesktopData *sdd;
+	GdkScreen       *screen;
+	
+	sdd = (ShowDesktopData *) data;
+
+	if (sdd->wnck_screen != NULL)
+		g_signal_handlers_disconnect_by_func (sdd->wnck_screen,
+						      show_desktop_changed_callback,
+						      sdd);
+
+	if (sdd->icon_theme != NULL)
+		g_signal_handlers_disconnect_by_func (sdd->icon_theme,
+						      theme_changed_callback,
+						      sdd);
+
+	screen = gtk_widget_get_screen (sdd->applet);
+	sdd->wnck_screen = wnck_screen_get (gdk_screen_get_number (screen));
+
+	if (sdd->wnck_screen != NULL)
+		wncklet_connect_while_alive (sdd->wnck_screen,
+					     "showing_desktop_changed",
+					     G_CALLBACK (show_desktop_changed_callback),
+					     sdd,
+					     sdd->applet);
+	else
+		g_warning ("Could not get WnckScreen!");
+
+        show_desktop_changed_callback (sdd->wnck_screen, sdd);
+
+	sdd->icon_theme = gtk_icon_theme_get_for_screen (screen);
+	wncklet_connect_while_alive (sdd->icon_theme, "changed",
+				     G_CALLBACK (theme_changed_callback),
+				     sdd,
+				     sdd->applet);
+
+        update_icon (sdd);
+}
+
+static void
+theme_changed_callback (GtkIconTheme    *icon_theme,
+			ShowDesktopData *sdd)
+{
+	update_icon (sdd);
+}
+
 gboolean
 show_desktop_applet_fill (PanelApplet *applet)
 {
         ShowDesktopData *sdd;
-        char *file;
-        GError *error;
-	AtkObject *atk_obj;
+	AtkObject       *atk_obj;
 
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 
@@ -293,30 +394,7 @@ show_desktop_applet_fill (PanelApplet *applet)
 
         sdd->applet = GTK_WIDGET (applet);
 
-	sdd->icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (sdd->icon_theme, TRUE);
-	file = gnome_icon_theme_lookup_icon (sdd->icon_theme,
-					     "gnome-fs-desktop", 48,
-					     NULL, NULL);
-        error = NULL;
-        if (file) {
-                sdd->icon = gdk_pixbuf_new_from_file (file, &error);
-                g_free (file);
-        }
-
-        if (sdd->icon == NULL) {
-                g_printerr (_("Failed to load %s: %s\n"),
-                            file, error ? error->message : _("File not found"));
-                if (error)
-                        g_error_free (error);
-        }
-
-        if (sdd->icon) {
-                sdd->image = gtk_image_new_from_pixbuf (sdd->icon);
-        } else {
-                sdd->image = gtk_image_new_from_stock (GTK_STOCK_MISSING_IMAGE,
-                                                       GTK_ICON_SIZE_SMALL_TOOLBAR);
-        }
+	sdd->image = gtk_image_new ();
 
         switch (panel_applet_get_orient (applet)) {
         case PANEL_APPLET_ORIENT_LEFT:
@@ -332,9 +410,22 @@ show_desktop_applet_fill (PanelApplet *applet)
 
         sdd->size = panel_applet_get_size (PANEL_APPLET (sdd->applet));
 
-        update_icon (sdd);
+	g_signal_connect (G_OBJECT (sdd->applet), "realize",
+			  G_CALLBACK (show_desktop_applet_realized), sdd);
 
         sdd->button = gtk_toggle_button_new ();
+
+	gtk_widget_set_name (sdd->button, "showdesktop-button");
+	gtk_rc_parse_string ("\n"
+			     "   style \"showdesktop-button-style\"\n"
+			     "   {\n"
+			     "      GtkWidget::focus-line-width=0\n"
+			     "      GtkWidget::focus-padding=0\n"
+			     "   }\n"
+			     "\n"
+			     "    widget \"*.showdesktop-button\" style \"showdesktop-button-style\"\n"
+			     "\n");
+
 	atk_obj = gtk_widget_get_accessible (sdd->button);
 	atk_object_set_name (atk_obj, _("Show Desktop Button"));
         g_signal_connect (G_OBJECT (sdd->button), "button_press_event",
@@ -347,19 +438,6 @@ show_desktop_applet_fill (PanelApplet *applet)
         gtk_container_add (GTK_CONTAINER (sdd->button), sdd->image);
         gtk_container_add (GTK_CONTAINER (sdd->applet), sdd->button);
 
-        update_button_state (sdd);
-
-        sdd->wnck_screen =
-                wnck_screen_get (gdk_screen_get_number (gtk_widget_get_screen (sdd->applet)));
-
-        if (sdd->wnck_screen != NULL)
-		wncklet_connect_while_alive (sdd->wnck_screen, "showing_desktop_changed",
-					     G_CALLBACK (show_desktop_changed_callback),
-					     sdd,
-					     sdd->applet);
-        else
-                g_warning ("Could not get WnckScreen!");
-
         /* FIXME: Update this comment. */
         /* we have to bind change_orient before we do applet_widget_add
            since we need to get an initial change_orient signal to set our
@@ -371,15 +449,12 @@ show_desktop_applet_fill (PanelApplet *applet)
 
         /* similiar to the above in semantics*/
         g_signal_connect (G_OBJECT (sdd->applet),
-                          "change_size",
-                          G_CALLBACK (applet_change_pixel_size),
+                          "size_allocate",
+                          G_CALLBACK (applet_size_allocated),
                           sdd);
 
-        /* FIXME: initial background, this needs some panel-applet voodoo */
-        g_signal_connect (G_OBJECT (sdd->applet),
-                          "change_background",
-                          G_CALLBACK (applet_change_background),
-                          sdd);
+	panel_applet_set_background_widget (PANEL_APPLET (sdd->applet),
+					    GTK_WIDGET (sdd->applet));
 
         panel_applet_setup_menu_from_file (PANEL_APPLET (sdd->applet),
                                            NULL,
@@ -393,7 +468,16 @@ show_desktop_applet_fill (PanelApplet *applet)
                           G_CALLBACK (applet_destroyed),
                           sdd);
 
-        gtk_widget_show_all (sdd->applet);
+	gtk_drag_dest_set (GTK_WIDGET(sdd->button), 0, NULL, 0, 0);
+
+	g_signal_connect (G_OBJECT(sdd->button), "drag_motion",
+			  G_CALLBACK (button_drag_motion),
+			  sdd);
+	g_signal_connect (G_OBJECT(sdd->button), "drag_leave",
+			  G_CALLBACK (button_drag_leave),
+			  sdd);
+
+  	gtk_widget_show_all (sdd->applet);
 
         return TRUE;
 }
@@ -403,32 +487,8 @@ display_help_dialog (BonoboUIComponent *uic,
                      ShowDesktopData         *sdd,
                      const gchar       *verbname)
 {
-        GError *error = NULL;
-
-        egg_help_display_desktop_on_screen (
-                NULL, "user-guide", "wgospanel.xml", "gospanel-564",
-                gtk_widget_get_screen (sdd->applet),
-		&error);
-
-        if (error) {
-                GtkWidget *dialog;
-                dialog = gtk_message_dialog_new (NULL,
-                                                 GTK_DIALOG_MODAL,
-                                                 GTK_MESSAGE_ERROR,
-                                                 GTK_BUTTONS_OK,
-                                                 _("There was an error displaying help: %s"),
-                                                 error->message);
-
-                g_signal_connect (G_OBJECT (dialog), "response",
-                                  G_CALLBACK (gtk_widget_destroy),
-                                  NULL);
-
-                gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-                gtk_window_set_screen (GTK_WINDOW (dialog),
-                                       gtk_widget_get_screen (sdd->applet));
-                gtk_widget_show (dialog);
-                g_error_free (error);
-        }
+	wncklet_display_help (sdd->applet, "user-guide",
+			      "user-guide.xml", "gospanel-564");
 }
 
 static void
@@ -441,38 +501,23 @@ display_about_dialog (BonoboUIComponent *uic,
                 NULL
         };
         static const char *documenters[] = {
+                "Sun GNOME Documentation Team <gdocteam@sun.com>",
                 NULL
         };
 
         /* Translator credits */
-        const char *translator_credits = _("translator_credits");
+        const char *translator_credits = _("translator-credits");
 
-        if (sdd->about_dialog) {
-                gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
-                                       gtk_widget_get_screen (sdd->applet));
-                gtk_window_present (GTK_WINDOW (sdd->about_dialog));
-                return;
-        }
-
-        sdd->about_dialog = gnome_about_new (_("Show Desktop Button"), VERSION,
-                                 "Copyright \xc2\xa9 2002 Red Hat, Inc.",
-                                 _("This button lets you hide all windows and show the desktop"),
-                                 authors,
-                                 documenters,
-                                 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-                                 sdd->icon);
-
-        gtk_window_set_wmclass (GTK_WINDOW (sdd->about_dialog), "show-desktop", "show-desktop");
-        gtk_window_set_screen (GTK_WINDOW (sdd->about_dialog),
-                               gtk_widget_get_screen (sdd->applet));
-
-        if (sdd->icon)
-                gtk_window_set_icon (GTK_WINDOW (sdd->about_dialog), sdd->icon);
-
-        g_signal_connect (G_OBJECT(sdd->about_dialog), "destroy",
-                          (GCallback)gtk_widget_destroyed, &sdd->about_dialog);
-
-        gtk_widget_show (sdd->about_dialog);
+	wncklet_display_about (sdd->applet, &sdd->about_dialog,
+			       _("Show Desktop Button"),
+			       "Copyright \xc2\xa9 2002 Red Hat, Inc.",
+			       _("This button lets you hide all windows and show the desktop."),
+			       authors,
+			       documenters,
+			       translator_credits,
+			       "gnome-fs-desktop",
+			       "show-desktop",
+			       "show-desktop");
 }
 
 static void
@@ -499,7 +544,7 @@ button_toggled_callback (GtkWidget       *button,
                                                  _("Your window manager does not support the show desktop button, or you are not running a window manager."));
 
                 g_object_add_weak_pointer (G_OBJECT (dialog),
-                                           (void**) &dialog);
+                                           (gpointer) &dialog);
                 
                 g_signal_connect (G_OBJECT (dialog), "response",
                                   G_CALLBACK (gtk_widget_destroy),
