@@ -21,13 +21,11 @@
 
 #include <stdlib.h>
 
-#include <gtk/gtk.h>
-#include <libbonobo.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
+#include <gtk/gtkdialog.h>
 #include <glade/glade-xml.h>
 #include <libwnck/libwnck.h>
 #include <gconf/gconf-client.h>
+#include <libgnomeui/gnome-help.h>
 
 #include "workspace-switcher.h"
 
@@ -79,8 +77,6 @@ static void display_help_dialog       (BonoboUIComponent *uic,
 static void display_about_dialog      (BonoboUIComponent *uic,
 				       PagerData         *pager,
 				       const gchar       *verbname);
-static void set_tooltip               (GtkWidget         *applet);
-
 
 static void
 pager_update (PagerData *pager)
@@ -95,26 +91,13 @@ pager_update (PagerData *pager)
 				 pager->display_all);
 }
 
-static WnckScreen *
-applet_get_screen (GtkWidget *applet)
-{
-	int screen_num;
-
-	if (!gtk_widget_has_screen (applet))
-		return wnck_screen_get_default ();
-
-	screen_num = gdk_screen_get_number (gtk_widget_get_screen (applet));
-
-	return wnck_screen_get (screen_num);
-}
-
 static void
 applet_realized (PanelApplet *applet,
 		 PagerData   *pager)
 {
 	WnckScreen *screen;
 
-	screen = applet_get_screen (GTK_WIDGET (applet));
+	screen = wncklet_get_screen (GTK_WIDGET (applet));
 
 	wnck_pager_set_screen (WNCK_PAGER (pager->pager), screen);
 }
@@ -147,6 +130,91 @@ applet_change_orient (PanelApplet       *applet,
 		gtk_label_set_text (GTK_LABEL (pager->label_row_col), pager->orientation == GTK_ORIENTATION_HORIZONTAL ? _("rows") : _("columns"));	
 }
 
+static void
+applet_change_background (PanelApplet               *applet,
+			  PanelAppletBackgroundType  type,
+			  GdkColor                  *color,
+			  GdkPixmap                 *pixmap,
+			  PagerData                 *pager)
+{
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+					    GTK_SHADOW_IN);
+		break;
+	case PANEL_COLOR_BACKGROUND:
+		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+					    GTK_SHADOW_NONE);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+					    GTK_SHADOW_NONE);
+		break;
+	}
+}
+
+static gboolean
+applet_scroll (PanelApplet    *applet,
+               GdkEventScroll *event,
+               PagerData      *pager)
+{
+	WnckScreen *screen;
+	int         index;
+	int         n_workspaces;
+	int         n_columns;
+	int         in_last_row;
+	
+	if (event->type != GDK_SCROLL)
+		return FALSE;
+
+	screen         = wncklet_get_screen (GTK_WIDGET (applet));
+	index          = wnck_workspace_get_number (wnck_screen_get_active_workspace (screen));
+	n_workspaces   = wnck_screen_get_workspace_count (screen);
+	n_columns      = n_workspaces / pager->n_rows;
+	if (n_workspaces % pager->n_rows != 0)
+		n_columns++;
+	in_last_row    = n_workspaces % n_columns;
+	
+	switch (event->direction) {
+	case GDK_SCROLL_DOWN:
+		if (index + n_columns < n_workspaces)
+			index += n_columns;
+		else if ((index < n_workspaces - 1
+			  && index + in_last_row != n_workspaces - 1) ||
+			 (index == n_workspaces - 1
+			  && in_last_row != 0))
+			index = (index % n_columns) + 1;
+		break;
+		
+	case GDK_SCROLL_RIGHT:
+		if (index < n_workspaces - 1)
+			index++;
+		break;
+		
+	case GDK_SCROLL_UP:
+		if (index - n_columns >= 0)
+			index -= n_columns;
+		else if (index > 0)
+			index = ((pager->n_rows - 1) * n_columns) + (index % n_columns) - 1;
+			if (index >= n_workspaces)
+				index -= n_columns;
+		break;
+
+	case GDK_SCROLL_LEFT:
+		if (index > 0)
+			index--;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	wnck_workspace_activate (wnck_screen_get_workspace (screen, index),
+				 event->time);
+	
+	return TRUE;
+}
+
 static void 
 response_cb (GtkWidget *widget,
 	     int        id,
@@ -399,7 +467,7 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 		break;
 	}
 
-	pager->screen = applet_get_screen (pager->applet);
+	pager->screen = wncklet_get_screen (pager->applet);
 
 	/* because the pager doesn't respond to signals at the moment */
 	wnck_screen_force_update (pager->screen);
@@ -413,10 +481,8 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 
 	pager_update (pager);
 	
-	gtk_widget_show (pager->pager);
 	gtk_container_add (GTK_CONTAINER (pager->applet), pager->pager);
-
-	set_tooltip (GTK_WIDGET (pager->pager));
+	gtk_widget_show (pager->pager);
 
 	gtk_widget_show (pager->applet);
 
@@ -428,6 +494,17 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
 			  pager);
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "scroll-event",
+			  G_CALLBACK (applet_scroll),
+			  pager);
+	g_signal_connect (G_OBJECT (pager->applet),
+			  "change_background",
+			  G_CALLBACK (applet_change_background),
+			  pager);
+
+	panel_applet_set_background_widget (PANEL_APPLET (pager->applet),
+					    GTK_WIDGET (pager->applet));
 	
 	panel_applet_setup_menu_from_file (PANEL_APPLET (pager->applet),
 					   NULL,
@@ -456,33 +533,8 @@ display_help_dialog (BonoboUIComponent *uic,
 		     PagerData         *pager,
 		     const gchar       *verbname)
 {
-	GError *error = NULL;
-
-	gnome_help_display_desktop_on_screen (
-		NULL, "workspace-switcher", "workspace-switcher", NULL,
-		gtk_widget_get_screen (pager->applet),
-		&error);
-
-	if (error) {
-		GtkWidget *dialog;
-		dialog = gtk_message_dialog_new (NULL,
-						 GTK_DIALOG_MODAL,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						  _("There was an error displaying help: %s"),
-						 error->message);
-
-		g_signal_connect (dialog, "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		gtk_window_set_screen (GTK_WINDOW (dialog),
-				       gtk_widget_get_screen (pager->applet));
-		gtk_widget_show (dialog);
-		g_error_free (error);
-
-	}
+	wncklet_display_help (pager->applet, "workspace-switcher",
+			      "workspace-switcher", NULL);
 }
 
 static void
@@ -490,9 +542,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		      PagerData         *pager,
 		      const gchar       *verbname)
 {
-	GdkPixbuf *pixbuf = NULL;
-	gchar *file;
-	
 	static const gchar *authors[] =
 	{
 		"Alexander Larsson <alla@lysator.liu.se>",
@@ -503,40 +552,18 @@ display_about_dialog (BonoboUIComponent *uic,
                 "Sun GNOME Documentation Team <gdocteam@sun.com>",
                 NULL
 	};
-	const char *translator_credits = _("translator_credits");
+	const char *translator_credits = _("translator-credits");
 
-	if (pager->about) {
-		gtk_window_set_screen (GTK_WINDOW (pager->about),
-				       gtk_widget_get_screen (pager->applet));
-		gtk_window_present (GTK_WINDOW (pager->about));
-		return;
-	}
-
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-workspace.png", TRUE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	g_free(file);
-
-	pager->about = gnome_about_new (_("Workspace Switcher"), VERSION,
-				 "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
-				 _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
-				 authors,
-				 documenters,
-				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
-	gtk_window_set_wmclass (GTK_WINDOW (pager->about), "pager", "Pager");
-	gtk_window_set_screen (GTK_WINDOW (pager->about),
-			       gtk_widget_get_screen (pager->applet));
-
-	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (pager->about), pixbuf);
-		g_object_unref (pixbuf);
-	}
-	
-	g_signal_connect (G_OBJECT(pager->about), "destroy",
-			  (GCallback)gtk_widget_destroyed, &pager->about);
-	
-	gtk_widget_show (pager->about);
+	wncklet_display_about (pager->applet, &pager->about,
+			       _("Workspace Switcher"),
+			       "Copyright \xc2\xa9 2001-2002 Red Hat, Inc.",
+			       _("The Workspace Switcher shows you a small version of your workspaces that lets you manage your windows."),
+			       authors,
+			       documenters,
+			       translator_credits,
+			       "gnome-workspace",
+			       "pager",
+			       "Pager");
 }
 
 
@@ -904,26 +931,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_object_unref (G_OBJECT (xml));
 	}
 
-	gnome_window_icon_set_from_file (GTK_WINDOW (pager->properties_dialog),
-	                                 GNOME_ICONDIR "/gnome-workspace.png");
+	gtk_window_set_icon_name (GTK_WINDOW (pager->properties_dialog),
+	                          "gnome-workspace");
 	gtk_window_set_screen (GTK_WINDOW (pager->properties_dialog),
 			       gtk_widget_get_screen (pager->applet));
 	gtk_window_present (GTK_WINDOW (pager->properties_dialog));
 }
-
-/*
- *Function to set tooltip
- */
-static void
-set_tooltip (GtkWidget *applet)
-{
-	GtkTooltips *tooltips;
-
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (tooltips);
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (applet), "tooltips", tooltips,
-                                (GDestroyNotify) g_object_unref);
-        gtk_tooltips_set_tip (tooltips, applet,
-                              _("Workspace Switcher"), NULL);
-}
