@@ -35,6 +35,7 @@
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
+#include "panel-separator.h"
 #include "panel-toplevel.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
@@ -48,11 +49,14 @@ typedef struct {
 
 	GtkWidget    *addto_dialog;
 	GtkWidget    *label;
+	GtkWidget    *search_entry;
 	GtkWidget    *back_button;
 	GtkWidget    *add_button;
 	GtkWidget    *tree_view;
 	GtkTreeModel *applet_model;
+	GtkTreeModel *filter_applet_model;
 	GtkTreeModel *application_model;
+	GtkTreeModel *filter_application_model;
 
 	GMenuTree    *menu_tree;
 
@@ -60,6 +64,9 @@ typedef struct {
 	GSList       *application_list;
 	GSList       *settings_list;
 
+	gchar        *search_text;
+	gchar        *applet_search_text;
+
 	guint         name_notify;
 
 	int           insertion_position;
@@ -75,6 +82,7 @@ typedef enum {
 	PANEL_ADDTO_LAUNCHER_NEW,
 	PANEL_ADDTO_MENU,
 	PANEL_ADDTO_MENUBAR,
+	PANEL_ADDTO_SEPARATOR,
 	PANEL_ADDTO_DRAWER
 } PanelAddtoItemType;
 
@@ -111,7 +119,7 @@ static PanelAddtoItemInfo special_addto_items [] = {
 
 	{ PANEL_ADDTO_LAUNCHER_MENU,
 	  N_("Application Launcher..."),
-	  N_("Launch a program that is already in the GNOME menu"),
+	  N_("Copy a launcher from the applications menu"),
 	  "launcher-program",
 	  PANEL_ACTION_NONE,
 	  NULL,
@@ -146,6 +154,17 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	  "MENUBAR:NEW",
 	  TRUE },
 
+	{ PANEL_ADDTO_SEPARATOR,
+	  N_("Separator"),
+	  N_("A separator to organize the panel items"),
+	  PANEL_SEPARATOR_ICON,
+	  PANEL_ACTION_NONE,
+	  NULL,
+	  NULL,
+	  NULL,
+	  "SEPARATOR:NEW",
+	  TRUE },
+
 	{ PANEL_ADDTO_DRAWER,
 	  N_("Drawer"),
 	  N_("A pop out drawer to store other items in"),
@@ -183,6 +202,9 @@ enum {
 
 static void panel_addto_present_applications (PanelAddtoDialog *dialog);
 static void panel_addto_present_applets      (PanelAddtoDialog *dialog);
+static gboolean panel_addto_filter_func (GtkTreeModel *model,
+					 GtkTreeIter  *iter,
+					 gpointer      data);
 
 static int
 panel_addto_applet_info_sort_func (PanelAddtoItemInfo *a,
@@ -217,13 +239,13 @@ panel_addto_prepend_internal_applets (GSList *list)
 		info              = g_new0 (PanelAddtoItemInfo, 1);
 		info->type        = PANEL_ADDTO_ACTION;
 		info->action_type = i;
-		info->name        = (char *) panel_action_get_text (i);
-		info->description = (char *) panel_action_get_tooltip (i);
-		info->icon        = (char *) panel_action_get_icon_name (i);
-		info->iid         = (char *) panel_action_get_drag_id (i);
-		info->static_data = TRUE;
+		info->name        = g_strdup (panel_action_get_text (i));
+		info->description = g_strdup (panel_action_get_tooltip (i));
+		info->icon        = g_strdup (panel_action_get_icon_name (i));
+		info->iid         = g_strdup (panel_action_get_drag_id (i));
+		info->static_data = FALSE;
 
-                list = g_slist_prepend (list, info);
+		list = g_slist_prepend (list, info);
 	}
 
         return list;
@@ -239,52 +261,26 @@ panel_addto_make_text (const char *name,
 	real_name = name ? name : _("(empty)");
 
 	if (!string_empty (desc)) {
-		result = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>\n%s",
+		result = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>\n%s",
 						  real_name, desc);
 	} else {
-		result = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>",
+		result = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
 						  real_name);
 	}
 
 	return result;
 }
 
+#define ICON_SIZE 32
+
 static GdkPixbuf *
-panel_addto_make_pixbuf (const char *filename,
-			 GtkIconSize size)
+panel_addto_make_pixbuf (const char *filename)
 {
-	GdkPixbuf *pb, *newpb;
-	int width, height;
-	int desired_width, desired_height;
-
-	if (!gtk_icon_size_lookup (size, &desired_width, &desired_height))
-		return NULL;
-
-	pb = panel_load_icon (gtk_icon_theme_get_default (), filename,
-			      desired_height, NULL);
-	if (!pb)
-		return NULL;
-
-	width = gdk_pixbuf_get_width (pb);
-	height = gdk_pixbuf_get_height (pb);
-
-	/* If the icon is larger than the icon size, then scale down
-	 * to fit in the bounding box. */
-	if (height > desired_height || width > desired_width) {
-		if (width * desired_height / height > desired_width)
-			desired_height = height * desired_width / width;
-		else
-			desired_width = width * desired_height / height;
-
-		newpb = gdk_pixbuf_scale_simple (pb,
-						 desired_width,
-						 desired_height,
-						 GDK_INTERP_BILINEAR);
-		g_object_unref (pb);
-		pb = newpb;
-	}
-
-	return pb;
+	//FIXME: size shouldn't be fixed but should depend on the font size
+	return panel_load_icon (gtk_icon_theme_get_default (),
+				filename,
+				ICON_SIZE, ICON_SIZE, ICON_SIZE,
+				NULL);
 }
 
 static void  
@@ -305,18 +301,23 @@ panel_addto_drag_begin_cb (GtkWidget      *widget,
 			   GdkDragContext *context,
 			   gpointer        data)
 {
-	GtkTreeModel *model;
+	GtkTreeModel *filter_model;
+	GtkTreeModel *child_model;
 	GtkTreePath  *path;
 	GtkTreeIter   iter;
+	GtkTreeIter   filter_iter;
 	GdkPixbuf    *pixbuf;
 
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
+	filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 	   
 	gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &path, NULL);
-	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get_iter (filter_model, &filter_iter, path);
 	gtk_tree_path_free (path);
+	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
+							  &iter, &filter_iter);
 
-	gtk_tree_model_get (model, &iter,
+	child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+	gtk_tree_model_get (child_model, &iter,
 	                    COLUMN_ICON, &pixbuf,
 	                    -1);
 
@@ -433,7 +434,8 @@ panel_addto_query_applets (GSList *list)
 						       "panel:icon",
 						       NULL);
 
-		if (!name) {
+		if (!name ||
+		    panel_lockdown_is_applet_disabled (info->iid)) {
 			continue;
 		}
 
@@ -475,8 +477,7 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 		pixbuf = NULL;
 
 		if (applet->icon != NULL) {
-			pixbuf = panel_addto_make_pixbuf (applet->icon,
-							  GTK_ICON_SIZE_DIALOG);
+			pixbuf = panel_addto_make_pixbuf (applet->icon);
 		}
 
 		gtk_list_store_append (model, &iter);
@@ -517,12 +518,15 @@ panel_addto_append_special_applets (PanelAddtoDialog *dialog,
 	translated = TRUE;
 }
 
-static GtkTreeModel *
+static void
 panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 {
 	GtkListStore *model;
 	GSList       *l;
 
+	if (dialog->filter_applet_model != NULL)
+		return;
+
 	if (panel_profile_id_lists_are_writable ()) {
 		dialog->applet_list = panel_addto_query_applets (dialog->applet_list);
 		dialog->applet_list = panel_addto_prepend_internal_applets (dialog->applet_list);
@@ -546,7 +550,12 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 	for (l = dialog->applet_list; l; l = l->next)
 		panel_addto_append_item (dialog, model, l->data);
 
-	return (GtkTreeModel *) model;
+	dialog->applet_model = GTK_TREE_MODEL (model);
+	dialog->filter_applet_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (dialog->applet_model),
+								 NULL);
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (dialog->filter_applet_model),
+						panel_addto_filter_func,
+						dialog, NULL);
 }
 
 static void panel_addto_make_application_list (GSList             **parent_list,
@@ -684,8 +693,7 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 
 		text = panel_addto_make_text (data->item_info.name,
 					      data->item_info.description);
-		pixbuf = panel_addto_make_pixbuf (data->item_info.icon,
-						  GTK_ICON_SIZE_DIALOG);
+		pixbuf = panel_addto_make_pixbuf (data->item_info.icon);
 		gtk_tree_store_set (store, &iter,
 				    COLUMN_ICON, pixbuf,
 				    COLUMN_TEXT, text,
@@ -705,13 +713,16 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 	}
 }
 
-static GtkTreeModel *
+static void
 panel_addto_make_application_model (PanelAddtoDialog *dialog)
 {
 	GtkTreeStore      *store;
 	GMenuTree          *tree;
 	GMenuTreeDirectory *root;
 
+	if (dialog->filter_application_model != NULL)
+		return;
+
 	store = gtk_tree_store_new (NUMBER_COLUMNS,
 				    GDK_TYPE_PIXBUF,
 				    G_TYPE_STRING,
@@ -753,19 +764,20 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 
 	gmenu_tree_unref (tree);
 
-	return GTK_TREE_MODEL (store);
+	dialog->application_model = GTK_TREE_MODEL (store);
+	dialog->filter_application_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (dialog->application_model),
+								      NULL);
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (dialog->filter_application_model),
+						panel_addto_filter_func,
+						dialog, NULL);
 }
 
-static gboolean
+static void
 panel_addto_add_item (PanelAddtoDialog   *dialog,
 	 	      PanelAddtoItemInfo *item_info)
 {
-	gboolean destroy;
-	
 	g_assert (item_info != NULL);
 
-	destroy = TRUE;
-
 	switch (item_info->type) {
 	case PANEL_ADDTO_APPLET:
 		panel_applet_frame_create (dialog->panel_widget->toplevel,
@@ -779,7 +791,6 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 		break;
 	case PANEL_ADDTO_LAUNCHER_MENU:
 		panel_addto_present_applications (dialog);
-		destroy = FALSE;
 		break;
 	case PANEL_ADDTO_LAUNCHER:
 		panel_launcher_create (dialog->panel_widget->toplevel,
@@ -802,14 +813,16 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 		panel_menu_bar_create (dialog->panel_widget->toplevel,
 				       dialog->insertion_position);
 		break;
+	case PANEL_ADDTO_SEPARATOR:
+		panel_separator_create (dialog->panel_widget->toplevel,
+					dialog->insertion_position);
+		break;
 	case PANEL_ADDTO_DRAWER:
 		panel_drawer_create (dialog->panel_widget->toplevel,
 				     dialog->insertion_position,
 				     NULL, FALSE, NULL);
 		break;
 	}
-
-	return destroy;
 }
 
 static void
@@ -818,28 +831,32 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 			     PanelAddtoDialog *dialog)
 {
 	GtkTreeSelection *selection;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
+	GtkTreeModel     *filter_model;
+	GtkTreeModel     *child_model;
+	GtkTreeIter       iter;
+	GtkTreeIter       filter_iter;
 
 	switch (response_id) {
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (dialog->addto_dialog)),
-				 "user-guide.xml", "gospanel-28"); //FIXME
+				 "user-guide.xml", "gospanel-15");
 		break;
 
 	case PANEL_ADDTO_RESPONSE_ADD:
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->tree_view));
-		if (gtk_tree_selection_get_selected (selection,
-						     &model, &iter)) {
+		if (gtk_tree_selection_get_selected (selection, &filter_model,
+						     &filter_iter)) {
 			PanelAddtoItemInfo *data;
 
-			gtk_tree_model_get (model, &iter,
+			gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
+									  &iter,
+									  &filter_iter);
+			child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+			gtk_tree_model_get (child_model, &iter,
 					    COLUMN_DATA, &data, -1);
 
-			if (data != NULL) {
-				if (panel_addto_add_item (dialog, data))
-					gtk_widget_destroy (widget_dialog);
-			}
+			if (data != NULL)
+				panel_addto_add_item (dialog, data);
 		}
 		break;
 
@@ -850,7 +867,7 @@ panel_addto_dialog_response (GtkWidget *widget_dialog,
 		panel_addto_present_applets (dialog);
 		break;
 
-	case GTK_RESPONSE_CANCEL:
+	case GTK_RESPONSE_CLOSE:
 		gtk_widget_destroy (widget_dialog);
 		break;
 
@@ -872,25 +889,42 @@ panel_addto_dialog_destroy (GtkWidget *widget_dialog,
 static void
 panel_addto_present_applications (PanelAddtoDialog *dialog)
 {
-	if (dialog->application_model == NULL)
-		dialog->application_model = panel_addto_make_application_model (dialog);
+	if (dialog->filter_application_model == NULL)
+		panel_addto_make_application_model (dialog);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->tree_view),
-				 dialog->application_model);
+				 dialog->filter_application_model);
 	gtk_window_set_focus (GTK_WINDOW (dialog->addto_dialog),
-			      dialog->tree_view);
+			      dialog->search_entry);
 	gtk_widget_set_sensitive (dialog->back_button, TRUE);
+
+	if (dialog->applet_search_text)
+		g_free (dialog->applet_search_text);
+
+	dialog->applet_search_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->search_entry)));
+	/* show everything */
+	gtk_entry_set_text (GTK_ENTRY (dialog->search_entry), "");
 }
 
 static void
 panel_addto_present_applets (PanelAddtoDialog *dialog)
 {
-	if (dialog->applet_model == NULL)
-		dialog->applet_model = panel_addto_make_applet_model (dialog);
+	if (dialog->filter_applet_model == NULL)
+		panel_addto_make_applet_model (dialog);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (dialog->tree_view),
-				 dialog->applet_model);
+				 dialog->filter_applet_model);
 	gtk_window_set_focus (GTK_WINDOW (dialog->addto_dialog),
-			      dialog->tree_view);
+			      dialog->search_entry);
 	gtk_widget_set_sensitive (dialog->back_button, FALSE);
+
+	if (dialog->applet_search_text) {
+		gtk_entry_set_text (GTK_ENTRY (dialog->search_entry),
+				    dialog->applet_search_text);
+		gtk_editable_set_position (GTK_EDITABLE (dialog->search_entry),
+					   -1);
+
+		g_free (dialog->applet_search_text);
+		dialog->applet_search_text = NULL;
+	}
 }
 
 static void
@@ -960,6 +994,14 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 		gconf_client_notify_remove (client, dialog->name_notify);
 	dialog->name_notify = 0;
 
+	if (dialog->search_text)
+		g_free (dialog->search_text);
+	dialog->search_text = NULL;
+
+	if (dialog->applet_search_text)
+		g_free (dialog->applet_search_text);
+	dialog->applet_search_text = NULL;
+
 	if (dialog->addto_dialog)
 		gtk_widget_destroy (dialog->addto_dialog);
 	dialog->addto_dialog = NULL;
@@ -978,10 +1020,18 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 	panel_addto_dialog_free_application_list (dialog->application_list);
 	panel_addto_dialog_free_application_list (dialog->settings_list);
 
+	if (dialog->filter_applet_model)
+		g_object_unref (dialog->filter_applet_model);
+	dialog->filter_applet_model = NULL;
+
 	if (dialog->applet_model)
 		g_object_unref (dialog->applet_model);
 	dialog->applet_model = NULL;
 
+	if (dialog->filter_application_model)
+		g_object_unref (dialog->filter_application_model);
+	dialog->filter_application_model = NULL;
+
 	if (dialog->application_model)
 		g_object_unref (dialog->application_model);
 	dialog->application_model = NULL;
@@ -1003,17 +1053,17 @@ panel_addto_name_change (PanelAddtoDialog *dialog,
 	label = NULL;
 
 	if (!string_empty (name))
-		label = g_strdup_printf (_("Select an _item to add to \"%s\":"),
+		label = g_strdup_printf (_("Find an _item to add to \"%s\":"),
 					 name);
 
 	if (panel_toplevel_get_is_attached (dialog->panel_widget->toplevel)) {
 		title = g_strdup_printf (_("Add to Drawer"));
 		if (label == NULL)
-			label = g_strdup (_("Select an _item to add to the drawer:"));
+			label = g_strdup (_("Find an _item to add to the drawer:"));
 	} else {
 		title = g_strdup_printf (_("Add to Panel"));
 		if (label == NULL)
-			label = g_strdup (_("Select an _item to add to the panel:"));
+			label = g_strdup (_("Find an _item to add to the panel:"));
 	}
 
 	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog), title);
@@ -1047,24 +1097,108 @@ panel_addto_name_notify (GConfClient      *client,
 		panel_addto_name_change (dialog, text);
 }
 
+static gboolean
+panel_addto_filter_func (GtkTreeModel *model,
+			 GtkTreeIter  *iter,
+			 gpointer      userdata)
+{
+	PanelAddtoDialog   *dialog;
+	PanelAddtoItemInfo *data;
+
+	dialog = (PanelAddtoDialog *) userdata;
+
+	if (!dialog->search_text || !dialog->search_text[0])
+		return TRUE;
+
+	gtk_tree_model_get (model, iter, COLUMN_DATA, &data, -1);
+
+	if (data == NULL)
+		return FALSE;
+
+	/* This is more a workaround than anything else: show all the root
+	 * items in a tree store */
+	if (GTK_IS_TREE_STORE (model) &&
+	    gtk_tree_store_iter_depth (GTK_TREE_STORE (model), iter) == 0)
+		return TRUE;
+
+	return (panel_util_utf8_strstrcase (data->name,
+					    dialog->search_text) != NULL ||
+	        panel_util_utf8_strstrcase (data->description,
+					    dialog->search_text) != NULL);
+}
+
+static void
+panel_addto_search_entry_changed (GtkWidget        *entry,
+				  PanelAddtoDialog *dialog)
+{
+	GtkTreeModel *model;
+	char         *new_text;
+	GtkTreeIter   iter;
+	GtkTreePath  *path;
+
+	new_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (dialog->search_entry)));
+	g_strchomp (new_text);
+		
+	if (dialog->search_text &&
+	    g_utf8_collate (new_text, dialog->search_text) == 0) {
+		g_free (new_text);
+		return;
+	}
+
+	if (dialog->search_text)
+		g_free (dialog->search_text);
+	dialog->search_text = new_text;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->tree_view));
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (model));
+
+	path = gtk_tree_path_new_first ();
+	if (gtk_tree_model_get_iter (model, &iter, path)) {
+		GtkTreeSelection *selection;
+
+		gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (dialog->tree_view),
+					      path, NULL, FALSE, 0, 0);
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->tree_view));
+		gtk_tree_selection_select_path (selection, path);
+	}
+	gtk_tree_path_free (path);
+}
+
+static void
+panel_addto_search_entry_activated (GtkWidget        *entry,
+				    PanelAddtoDialog *dialog)
+{
+	gtk_dialog_response (GTK_DIALOG (dialog->addto_dialog),
+			     PANEL_ADDTO_RESPONSE_ADD);
+}
+
 static void
 panel_addto_selection_changed (GtkTreeSelection *selection,
 			       PanelAddtoDialog *dialog)
 {
-	GtkTreeModel       *model;
+	GtkTreeModel       *filter_model;
+	GtkTreeModel       *child_model;
 	GtkTreeIter         iter;
+	GtkTreeIter         filter_iter;
 	PanelAddtoItemInfo *data;
 	char               *iid;
 
-	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
-		gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
+	if (!gtk_tree_selection_get_selected (selection,
+					      &filter_model,
+					      &filter_iter)) {
+		gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button),
+					  FALSE);
 		return;
 	}
 
-	gtk_tree_model_get (model, &iter, COLUMN_DATA, &data, -1);
+	gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
+							  &iter, &filter_iter);
+	child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+	gtk_tree_model_get (child_model, &iter, COLUMN_DATA, &data, -1);
 
 	if (!data) {
-		gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
+		gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button),
+					  FALSE);
 		return;
 	}
 
@@ -1148,6 +1282,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	PanelAddtoDialog *dialog;
 	GtkWidget *vbox;
 	GtkWidget *inner_vbox;
+	GtkWidget *find_hbox;
 	GtkWidget *sw;
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
@@ -1175,12 +1310,12 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	dialog->back_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_GO_BACK,
 						     PANEL_ADDTO_RESPONSE_BACK);
-	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
-			       GTK_STOCK_CANCEL,
-			       GTK_RESPONSE_CANCEL);
 	dialog->add_button = gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
 						     GTK_STOCK_ADD,
 						     PANEL_ADDTO_RESPONSE_ADD);
+	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
+			       GTK_STOCK_CLOSE,
+			       GTK_RESPONSE_CLOSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->addto_dialog),
 				      FALSE);
@@ -1202,13 +1337,28 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	inner_vbox = gtk_vbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (vbox), inner_vbox, TRUE, TRUE, 0);
 
+	find_hbox = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (inner_vbox), find_hbox, FALSE, FALSE, 0);
+
 	dialog->label = gtk_label_new_with_mnemonic ("");
 	gtk_misc_set_alignment (GTK_MISC (dialog->label), 0.0, 0.5);
 	gtk_label_set_use_markup (GTK_LABEL (dialog->label), TRUE);
 
-	gtk_box_pack_start (GTK_BOX (inner_vbox), dialog->label,
+	gtk_box_pack_start (GTK_BOX (find_hbox), dialog->label,
 			    FALSE, FALSE, 0);
 
+	dialog->search_entry = gtk_entry_new (); 
+	g_signal_connect (G_OBJECT (dialog->search_entry), "changed",
+			  G_CALLBACK (panel_addto_search_entry_changed), dialog);
+	g_signal_connect (G_OBJECT (dialog->search_entry), "activate",
+			  G_CALLBACK (panel_addto_search_entry_activated), dialog);
+
+	gtk_box_pack_end (GTK_BOX (find_hbox), dialog->search_entry,
+			  TRUE, TRUE, 0);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (dialog->label),
+				       dialog->search_entry);
+
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC,
@@ -1240,6 +1390,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 						     "markup", COLUMN_TEXT,
 						     NULL);
 
+	//FIXME use the same search than the one for the search entry?
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dialog->tree_view),
 					 COLUMN_SEARCH);
 
@@ -1268,10 +1419,10 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	gtk_widget_show_all (vbox);
 
-	gtk_label_set_mnemonic_widget (GTK_LABEL (dialog->label),
-				       dialog->tree_view);
-
 	panel_toplevel_push_autohide_disabler (dialog->panel_widget->toplevel);
+	panel_widget_register_open_dialog (panel_widget,
+					   dialog->addto_dialog);
+
 	panel_addto_name_change (dialog,
 				 panel_toplevel_get_name (dialog->panel_widget->toplevel));
 
