@@ -28,9 +28,11 @@
 
 #include <string.h>
 #include <gdk/gdkx.h>
+#include <cairo.h>
+#include <cairo-xlib.h> //FIXME should be removed when gdk_cairo_set_source_pixmap() is available (GTK+ 2.10)
 
 #include "panel-background-monitor.h"
-#include "panel-gdk-pixbuf-extensions.h"
+#include "panel-util.h"
 
 
 static gboolean panel_background_composite (PanelBackground *background);
@@ -146,6 +148,8 @@ panel_background_prepare (PanelBackground *background)
 
 	background->prepared = TRUE;
 
+	background->notify_changed (background, background->user_data);
+
 	return TRUE;
 }
 
@@ -211,10 +215,12 @@ get_desktop_pixbuf (PanelBackground *background)
 static GdkPixbuf *
 composite_image_onto_desktop (PanelBackground *background)
 {
-	GdkPixbuf *retval;
-	ArtIRect   rect;
-	int        width, height;
-	int        tilewidth, tileheight;
+	GdkPixbuf       *retval;
+	int              width, height;
+	unsigned char   *data;
+	cairo_t         *cr;
+	cairo_surface_t *surface;
+	cairo_pattern_t *pattern;
 
 	if (!background->desktop)
 		background->desktop = get_desktop_pixbuf (background);
@@ -222,23 +228,37 @@ composite_image_onto_desktop (PanelBackground *background)
 	if (!background->desktop)
 		return NULL;
 
-	retval = gdk_pixbuf_copy (background->desktop);
-
 	width  = gdk_pixbuf_get_width  (background->desktop);
 	height = gdk_pixbuf_get_height (background->desktop);
 
-	tilewidth  = gdk_pixbuf_get_width (background->transformed_image);
-	tileheight = gdk_pixbuf_get_height (background->transformed_image);
+	data = g_malloc (width * height * 4);
+	if (!data)
+		return NULL;
 
-	rect.x0 = 0;
-	rect.y0 = 0;
-	rect.x1 = width;
-	rect.y1 = height;
+	surface = cairo_image_surface_create_for_data (data,
+						       CAIRO_FORMAT_RGB24,
+						       width, height,
+						       width * 4);
+	cr = cairo_create (surface);
+	cairo_set_source_rgb (cr, 1, 1, 1);
+	cairo_paint (cr);
 
-	panel_gdk_pixbuf_draw_to_pixbuf_tiled (
-		background->transformed_image,
-		retval, rect, tilewidth, tileheight,
-		0, 0, 255, GDK_INTERP_NEAREST);
+	gdk_cairo_set_source_pixbuf (cr, background->desktop, 0, 0);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_fill (cr);
+
+	gdk_cairo_set_source_pixbuf (cr, background->transformed_image, 0, 0);
+	pattern = cairo_get_source (cr);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_fill (cr);
+
+	cairo_destroy (cr);
+	cairo_surface_destroy (surface);
+
+	retval = panel_util_cairo_rgbdata_to_pixbuf (data, width, height);
+
+	g_free (data);
 
 	return retval;
 }
@@ -326,8 +346,6 @@ panel_background_composite (PanelBackground *background)
 
 	panel_background_prepare (background);
 
-	background->notify_changed (background, background->user_data);
-
 	return TRUE;
 }
 
@@ -477,7 +495,7 @@ panel_background_transform (PanelBackground *background)
 
 	free_transformed_resources (background);
 
-	if (background->type == PANEL_BACK_IMAGE) 
+	if (background->type == PANEL_BACK_IMAGE)
 		background->transformed_image =
 			get_scaled_and_rotated_pixbuf (background);
 
@@ -530,6 +548,8 @@ load_background_file (PanelBackground *background)
 	if (!g_file_test (background->image, G_FILE_TEST_IS_REGULAR))
 		return;
 
+	//FIXME add a monitor on the file so that we reload the background
+	//when it changes
 	background->loaded_image = 
 		gdk_pixbuf_new_from_file (background->image, &error);
 	if (!background->loaded_image) {
@@ -538,6 +558,8 @@ load_background_file (PanelBackground *background)
 			   background->image, error->message);
 		g_error_free (error);
 	}
+
+	panel_background_update_has_alpha (background);
 }
 
 void
@@ -1033,3 +1055,116 @@ panel_background_effective_type (PanelBackground *background)
 
 	return retval;
 }
+
+static void
+panel_background_set_no_background_on_widget (PanelBackground *background,
+					      GtkWidget       *widget)
+{
+	GtkRcStyle *rc_style;
+
+	gtk_widget_set_style (widget, NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (widget, rc_style);
+	gtk_rc_style_unref (rc_style);
+}
+
+static void
+panel_background_set_image_background_on_widget (PanelBackground *background,
+						 GtkWidget       *widget)
+{
+	const GdkPixmap *bg_pixmap;
+	GdkPixmap       *pixmap;
+	int              width;
+	int              height;
+	cairo_t         *cr;
+	cairo_surface_t *surface;
+	cairo_pattern_t *pattern;
+	cairo_matrix_t   matrix;
+	GtkStyle         *style;
+
+	bg_pixmap = panel_background_get_pixmap (background);
+	if (!bg_pixmap)
+		return;
+
+	gdk_drawable_get_size (GDK_DRAWABLE (bg_pixmap), &width, &height);
+
+	//FIXME gdk_cairo_set_source_pixmap() should do the trick (GTK+ 2.10)
+	surface = cairo_xlib_surface_create (gdk_x11_drawable_get_xdisplay (widget->window),
+ 					     gdk_x11_drawable_get_xid (GDK_DRAWABLE (bg_pixmap)),
+					     gdk_x11_visual_get_xvisual (gdk_drawable_get_visual (widget->window)),
+					     width, height);
+
+	pattern = cairo_pattern_create_for_surface (surface);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+	cairo_matrix_init_translate (&matrix,
+				     widget->allocation.x,
+				     widget->allocation.y);
+	cairo_pattern_set_matrix (pattern, &matrix);
+
+	pixmap = gdk_pixmap_new (widget->window,
+				 widget->allocation.width,
+				 widget->allocation.height,
+				 -1);
+
+	cr = gdk_cairo_create (GDK_DRAWABLE (pixmap));
+	cairo_set_source (cr, pattern);
+	cairo_rectangle (cr, 0, 0,
+			 widget->allocation.width, widget->allocation.height);
+	cairo_fill (cr);
+
+	cairo_pattern_destroy (pattern);
+	cairo_surface_destroy (surface);
+	cairo_destroy (cr);
+
+	style = gtk_style_copy (widget->style);
+	if (style->bg_pixmap[GTK_STATE_NORMAL])
+		g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+	style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+	gtk_widget_set_style (widget, style);
+	g_object_unref (style);
+
+	g_object_unref (pixmap);
+}
+
+static void
+panel_background_set_color_background_on_widget (PanelBackground *background,
+						 GtkWidget       *widget)
+{
+	const PanelColor *color;
+
+	color = panel_background_get_color (background);
+	if (color->alpha != 0xffff) {
+		panel_background_set_image_background_on_widget (background,
+								 widget);
+		return;
+	}
+
+	gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, &color->gdk);
+}
+
+void
+panel_background_change_background_on_widget (PanelBackground *background,
+					      GtkWidget       *widget)
+{
+	PanelBackgroundType type;
+
+	panel_background_set_no_background_on_widget (background, widget);
+
+	type = panel_background_get_type (background);
+
+	switch (type) {
+	case PANEL_BACK_NONE:
+		break;
+	case PANEL_BACK_COLOR:
+		panel_background_set_color_background_on_widget (background,
+								 widget);
+		break;
+	case PANEL_BACK_IMAGE:
+		panel_background_set_image_background_on_widget (background,
+								 widget);
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
